################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_check.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_check.py --help

Type::

   python graph_check.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

## patch for old python installations
if sys.version_info < (2,4):
    from sets import *
    set = Set

""" program $Id: graph_check.py 2782 2009-09-10 11:40:29Z andreas $

python graph_check.py < graph.in

Check graph for completeness.

"""
import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def writeSet( outfile, outset ):
    """write set to file."""
    outlist = list(outset)
    outlist.sort()
    for x in outlist: outfile.write( "%s\n" % x )

def writeInfo( outfile, vertices, nlinks, nlines, nerrors, ncomments, is_sorted ):
    
    all = set(vertices.keys())
    queries = set(filter( lambda x: vertices[x] & 1, all) )
    sbjcts  = set(filter( lambda x: vertices[x] & 2, all) )
    ## count only those as missed self, that do have queries.
    missed_self = set(filter( lambda x: vertices[x] & 4 == 0, all) ).intersection( queries )

    missed_queries = all.difference( queries)
    missed_sbjcts =  all.difference( sbjcts )

    if is_sorted:
        sorted= "yes"
    else:
        sorted = "no"

    outfile.write( "\t".join( map(str, (len(queries), len(sbjcts),
                                               len(queries.union( sbjcts ) ),
                                               nlinks,
                                               nlines, nerrors, ncomments,
                                               sorted,
                                               len(all),
                                               len(missed_queries),
                                               len(missed_sbjcts),
                                               len(missed_self) ) ) ) + "\n") 
    outfile.flush()

    return missed_queries, missed_sbjcts, missed_self
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: graph_check.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("--filename-missing", dest="filename_missing", type="string",
                      help="missing entries.")
    parser.add_option("--filename-found", dest="filename_found", type="string",
                      help="found entries.")
    parser.add_option("--report-step1", dest="report_step1", type="int",
                      help="report interval for input.")
    parser.add_option("--report-step2", dest="report_step2", type="int",
                      help="report interval for processing.")
    parser.add_option("-n", "--filename-vertices", dest="filename_vertices", type="string",
                      help="filename with vertices.")
    parser.add_option("-u", "--num-fields", dest="num_fields", type="int",
                      help="number of fields to expect.")
    parser.add_option("-o", "--filename-output-pattern", dest="filename_output_pattern", type="string",
                      help="filenames for output (should contain one %s for one section).")
    parser.add_option("-s", "--sort-order", dest="sort_order", type="choice",
                      choices=("numeric", "alphanumeric" ),
                      help="sort order - if numeric, vertices are cast to int.")

    parser.set_defaults(
        filename_vertices = None,
        report_step1 = 100000,
        report_step2 = 10000,
        filename_output_pattern = "%s",
        subsets = False,
        num_fields = 11,
        sort_order = "alphanumeric",
        )

    (options, args) = E.Start( parser )

    if options.loglevel >= 1:
        options.stdlog.write("# output goes to:\n" )
        options.stdlog.write("# errors: %s\n" % options.filename_output_pattern % "errors" )
        options.stdlog.write("# missed query: %s\n" % options.filename_output_pattern % "missed_queries" )
        options.stdlog.write("# missed sbjct: %s\n" % options.filename_output_pattern % "missed_sbjcts" )
        options.stdlog.write("# missed self: %s\n" % options.filename_output_pattern % "missed_self" )                
        
    outfile_errors = open( options.filename_output_pattern % "errors", "w" )

    if options.sort_order == "numeric":
        f = int
    else:
        f= str

    if options.filename_vertices:
        vv, errors = IOTools.ReadList( open( options.filename_vertices, "r" ), map_function = f )
        vertices = {}
        ## use flags for vertices
        ## 1st bit: is query: 1
        ## 2nd bit: is sbjct: 2
        ## 3rd bit: has self: 4
        for v in vv:
            vertices[v] = 0
    else:
        raise "for the time being, specify a vertex file."

    options.stdout.write( "nqueries\tnsbjcts\tnvertices\tnlinks\tnlines\tnerrors\tncomments\tis_sorted\tnexpected\tnmissed_queries\tnmissed_sbjcts\tnmissed_self\n" )

    ncomments, nlinks, nerrors, nlines = 0, 0, 0, 0

    is_sorted = True

    last = None

    
    for line in sys.stdin:

        nlines += 1
        if line[0] == "#":
            ncomments += 1
            continue

        nlinks += 1

        data = line[:-1].split("\t")

        if len(data) != options.num_fields:
            nerrors += 1
            outfile_errors.write( line )
            outfile_errors.flush()
            continue

        q, s = f(data[0]), f(data[1])

        if q == s: vertices[q] |= 4
        vertices[q] |= 1
        vertices[s] |= 2

        if last and last > q:
            is_sorted = False
            outfile_errors.write("# sort inconsistency between %s and %s at line %i\n" % ( last, q, nlines) )
            outfile_errors.flush()
            if options.loglevel >= 1:
                options.stdlog.write("# sort inconsistency between %s and %s at line %i\n" % ( last, q, nlines) )
                options.stdlog.flush()
                
        if options.report_step1 and nlines % options.report_step1 == 0:
            writeInfo( options.stdlog, vertices, nlinks, nlines, nerrors, ncomments, is_sorted )
            
        last = q

    missed_queries, missed_sbjcts, missed_self = writeInfo( options.stdout, vertices, nlinks, nlines, nerrors, ncomments, is_sorted )

    if nerrors == 0:
        os.remove( options.filename_output_pattern % "errors" )

    if missed_queries:
        writeSet( open( options.filename_output_pattern % "missed_queries", "w" ), missed_queries )

    if missed_sbjcts:
        writeSet( open( options.filename_output_pattern % "missed_sbjcts", "w" ), missed_sbjcts )

    if missed_self:        
        writeSet( open( options.filename_output_pattern % "missed_self", "w" ), missed_self )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2013 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat2rdf.py - create rdf description of CGAT script
====================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script creates an rdf description of a CGAT
script.

Optionally, the script outputs also a galaxy xml
description of the scripts' interface.

Usage
-----

Example::

   python cgat2rdf.py bam2stats.py 

Type::

   python cgat2rdf.py --help

for command line help.

Documentation
-------------

This script takes a CGAT script and attempts to write an interface definition
for this script. In order to guess the file types correctly, :file:`cgat2rdf.py`
makes use of the following information:

1. The script name. If the name of the script contains a "format2format.py", :file:`cgat2rdf.py`
   will assume that the script works within a pipe: it takes stdin and stdout as 
   input and output, respectively, and each are formatted according to the formats.
   For example, ``bed2bed.py`` has a :term:`bed` formatted file as input and output,
   while ``gtf2gff.py`` has a :term:`gtf` formatted file as input and outputs
   a :term:`gff` file. Most formats are not parsed, though :file:`cgat2rdf.py` contains
   some type mappings:

+--------------------+--------------------+--------------------+
|Format              |Maps to             |Content             |
+--------------------+--------------------+--------------------+
|tsv                 |tabular             |Tab-separated values|
+--------------------+--------------------+--------------------+
|table               |tabular             |ditto               |
+--------------------+--------------------+--------------------+
|stats               |tabular             |ditto               |
+--------------------+--------------------+--------------------+
|csv                 |tabular             |ditto               |
+--------------------+--------------------+--------------------+

   
2. The command line options. :file:`cgat2rdf.py` will import the script it 
   runs and captures the command line option parser information. Based on these
   data, options are added to the interface. Atomic values such as int, float,
   etc, are interpreted directly. For textual arguments, :file:`cgat2rdf.py`
   tests if the :attr:`metavar` attribute has been set. When set, the content
   of this attribute will determine the file type.

The interface decription can be exported either as :term:`RDF` or in a variety
of other formats:

galaxy
    Galaxy xml file.

Code
----

'''

import os
import sys
import re
import optparse
import datetime
import collections

from jinja2 import Template

import CGAT.Experiment as E

# handle to original E.Start function
ORIGINAL_START = None

# Parser object collected from child script
PARSER = None

from rdflib import Graph
from rdflib import Namespace
from rdflib.namespace import RDF, RDFS, DCTERMS
from rdflib import Graph, Namespace, Literal, BNode, URIRef
from rdflib.collection import Collection

#DCTerms = Namespace('http://purl.org/dc/terms/')
FOAF = Namespace('http://xmlns.com/foaf/1.1/')
Component = Namespace('http://www.isi.edu/ikcap/Wingse/componentOntology.owl#')
FO = Namespace('http://www.isi.edu/ikcap/Wingse/fileOntology.owl#')
CLP = Namespace('http://www.humgen.nl/climate/ontologies/clp#')

def _e(string):
    return string.replace(' ', '_')

MAP_FORMATS = {
    'tsv' : 'tabular',
    'table' : 'tabular',
    'stats' : 'tabular',
    'csv' : 'tabular',
    }

MAP_TYPE2FORMAT = {
    'gff' : 'gff,gtf',
    'gtf' : 'gff,gtf',
    'bam' : 'bam',
    'sam' : 'sam',
    'bigwig': 'bigWig',
    'bed' : 'bed',
    }


class DummyError( Exception ): pass

class Generator:
    '''inspired by:
    https://github.com/zuotian/CLI-mate/blob/master/climate/utils/legacy_parser.py
    '''

    def __init__(self):
        self.graph = Graph()



    def _addTriple( self, s, p, o):
        if type(o) in [BNode, URIRef]:
            self.graph.add((s, p, o))
        elif type(o) is list:
            o_list = BNode()
            self.graph.add((s, p, o_list))
            os = Collection(self.graph, o_list)
            for item in o:
                os.append(Literal(item))
        elif o != '':
            self.graph.add((s, p, Literal(o)))
            
    def _generateStatements(self, subject_node, properties):
        """
        properties = {"from_loc" : ["data_table", "no"]
        "access_location : ["/path/to/somewhere/", "yes"]}
        """
        for (key, values) in properties.items():
            if values[1] == 'no': # not a volatile property.
                a_node = BNode()
                self._addTriple(a_node, RDF['type'], RDF['Statement'])
                self._addTriple(a_node, CLP['relatedTo'], Literal(key))
                self._addTriple(a_node, RDF['subject'], subject_node)
                self._addTriple(a_node, RDF['predicate'], CLP['hasProperty'])
                self._addTriple(a_node, RDF['object'], values[0])


    def _generateDependencies(self, ap_node, dependencies):
        if dependencies:
            for dep in dependencies:
                d_node = BNode()
                self._addTriple(d_node, RDF.type, CLP['dependency'])
                self._addTriple(d_node, CLP['hasDependingItem'], BNode(_e(dep['depending_parameter'])))
                self._addTriple(d_node, CLP['dependingCondition'], dep['depending_condition'])
                self._addTriple(d_node, CLP['hasDependentItem'], ap_node)
                self._addTriple(d_node, CLP['dependentScope'], dep['dependent_scope'])
                self._addTriple(d_node, CLP['effect'], dep['dependent_effect'])

    def _addDict(self, data ):
        '''convert a dictionary to an RDF graph.'''

        t_node = BNode(_e(data['name']))
        self._addTriple(t_node, RDF.type, CLP['CommandLineProgramComponentType'])
        self._addTriple(t_node, DCTERMS['label'], data['name'])
        self._addTriple(t_node, DCTERMS['title'], data['binary'])
        self._addTriple(t_node, DCTERMS['description'], data['description'])
        self._addTriple(t_node, Component['hasVersion'], data['version'])
        self._addTriple(t_node, DCTERMS['comment'], data['help'])
        self._generateStatements(t_node, data['property_bag'])

        r_node = BNode()
        self._addTriple(t_node, Component['hasExecutionRequirements'], r_node)
        self._addTriple(r_node, RDF.type, Component['ExecutionRequirements'])
        self._addTriple(r_node, Component['requiresOperationSystem'], Component['Linux']) # TODO
        if data['interpreter'] != '(binary)':
            self._addTriple(r_node, Component['requiresSoftware'], Component[data['interpreter']])
        if data['grid_access_type'] != '-':
            self._addTriple(r_node, CLP['gridAccessType'], data['grid_access_type'])
            self._addTriple(r_node, Component['gridID'], data['grid_access_location'])
        for req in data['requirements']:
            req_node = BNode()
            self._addTriple(r_node, CLP['requiresSoftware'], req_node)
            self._addTriple(req_node, RDF.type, CLP['Software'])
            self._addTriple(req_node, DCTERMS['title'], req['req_name'])
            self._addTriple(req_node, CLP['gridID'], req['req_location'])
            self._addTriple(req_node, CLP['softwareType'], req['req_type'])

        argument_list = BNode('argument_list')
        self._addTriple(t_node, Component['hasArguments'], argument_list)
        #self._addTriple(argument_list, RDF.type, Component['argumentAndPrefixList'])
        argument_nodes = Collection(self.graph, argument_list)

        input_list = BNode('input_list')
        self._addTriple(t_node, Component['hasInputs'], input_list)
        #self._addTriple(input_list, RDF.type, Component['FileOrCollectionList'])
        input_nodes = Collection(self.graph, input_list)

        output_list = BNode('output_list')
        self._addTriple(t_node, Component['hasOutputs'], output_list)
        #self._addTriple(output_list, RDF.type, Component['FileOrCollectionList'])
        output_nodes = Collection(self.graph, output_list)

        for p in data['parameters']:
            ap_node = BNode(_e(p['name']))
            argument_nodes.append(ap_node)
            self._addTriple(ap_node, RDF.type, Component['ArgumentAndPrefix'])

            a_node = BNode(_e(p['name']) + '_arg')
            self._addTriple(ap_node, Component['hasArgument'], a_node)

            choices = []
            if 'choices' in p and p['choices']:
                choices = map(lambda x: x.strip(), p['choices'].split(','))

            p_type = p['type']
            if p_type == 'integer':
                self._addTriple(a_node, RDF.type, FO['Int'])
                try:
                    self._addTriple(a_node, FO['hasIntValue'], int(p['value']))
                    choices = map(lambda x: int(x), choices)
                except ValueError:
                    pass # do nothing if value is not an integer
            elif p_type == 'float':
                self._addTriple(a_node, RDF.type, FO['Float'])
                try:
                    self._addTriple(a_node, FO['hasFloatValue'], float(p['value']))
                    choices = map(lambda x: float(x), choices)
                except ValueError:
                    pass # do nothing if value is not a float
            elif p_type in ['string', 'select']:
                self._addTriple(a_node, RDF.type, FO['String'])
                self._addTriple(a_node, FO['hasStringValue'], p['value'])
            elif p_type in ['input', 'stdin']:
                self._addTriple(a_node, RDF.type, FO['File'])
                self._addTriple(a_node, DCTERMS['format'], p['format'])
                self._addTriple(a_node, Component['hasValue'], p['value'])
                input_nodes.append(a_node)
            elif p_type in ['output', 'stdout', 'stderr']:
                self._addTriple(a_node, RDF.type, FO['File'])
                self._addTriple(a_node, DCTERMS['format'], p['format'])
                self._addTriple(a_node, Component['hasValue'], p['value'])
                output_nodes.append(a_node)
            else:
                self._addTriple(a_node, Component['hasValue'], p['value'])

            if choices:
                choices = map(lambda x: Literal(x), choices)
                choice_list = BNode(_e(p['name'] + '_choice_list'))
                choice_nodes = Collection(self.graph, choice_list, choices)
                self._addTriple(a_node, CLP['hasValueChoices'], choice_list)

            self._addTriple(ap_node, DCTERMS['title'], p['name'])
            self._addTriple(ap_node, DCTERMS['description'], p['description'])
            self._addTriple(ap_node, RDFS.label, p['label'])
            self._addTriple(ap_node, Component['hasPrefix'], p['arg'])
            self._addTriple(ap_node, CLP['hasAlternativePrefix'], p['arg_long'])
            self._addTriple(ap_node, CLP['order'], int(p['rank']))
            self._addTriple(ap_node, CLP['display'], p['display'])
            self._addTriple(ap_node, CLP['minOccurrence'], p['min_occurrence'])
            self._addTriple(ap_node, CLP['maxOccurrence'], p['max_occurrence'])

            self._generateStatements(ap_node, p['property_bag'])
            self._generateDependencies(ap_node, p['dependencies'])
        #for

    def _addMetaInfo(self, data):
        
        # meta data node about the Interface Generator itself.
        ig_node = URIRef('http://climate.host.url')
        self._addTriple(ig_node, RDF.type, FOAF['Agent'])
        self._addTriple(ig_node, DCTERMS['title'], data['meta_title'])
        self._addTriple(ig_node, DCTERMS['creator'], data['meta_title'])
        self._addTriple(ig_node, DCTERMS['hasVersion'], data['meta_title'])

        m_node = URIRef('')
        self._addTriple(m_node, RDF.type, FOAF['Document'])
        self._addTriple(m_node, DCTERMS['creator'], ig_node)
        self._addTriple(m_node, DCTERMS['created'], datetime.datetime.utcnow())
        self._addTriple(m_node, RDFS['label'], 'RDF Definition of ' + data['name'])

    def serialize(self, data, format='n3'):
        #TODO: current RDFLib doesn't support base url serialization!
        base_uri = "http://www.humgen.nl/climate/ontologies/clp" + _e(data['name']) + '.rdf#'
        Base = Namespace(base_uri)
        self.graph.bind('base', Base)

        self.graph.bind('dcterms', DCTERMS)
        self.graph.bind('foaf', FOAF)
        self.graph.bind('co', Component)
        self.graph.bind('fo', FO)
        self.graph.bind('clp', CLP)

        self._addDict(data)
        self._addMetaInfo(data)
        return self.graph.serialize(format=format)

def LocalStart( parser, **kwargs ):
    '''stub for E.Start - set return_parser argument to true'''
    global PARSER
    PARSER = ORIGINAL_START( parser,
                             return_parser = True,
                             **kwargs
                             )
    raise DummyError()

def getDescription( scriptname, docstring ):
    '''get script description from docstring.'''

    description = scriptname
    for line in docstring.split("\n"):
        if line.startswith( scriptname ):
            description = line[line.index("-")+1:].strip()
            break
        
    return description

def guessFormats( scriptname, docstring ):
    '''guess the input/output format of a script.'''

    input_format, output_format = "tsv", "tsv"

    if "2" in scriptname:
        input_format, output_format = scriptname.split("2")
        
    # map CGAT format names to GALAXY ones
    input_format = MAP_FORMATS.get( input_format, input_format )
    output_format = MAP_FORMATS.get( output_format, output_format )

    return input_format, output_format

def buildParam( **kwargs ):
    '''return a parameter with default values.

    Specific fields can be set by providing keyword arguments.
    '''

    param = {}

    param['label'] = "label"
    param['description'] = "description"
    param['rank'] = 1
    param['display'] = 'show'
    param['min_occurrence'] = 0
    param['max_occurrence'] = 1

    # get default value
    param['value'] = "value"
    param['type' ] = "text"
    param['dependencies'] = {}
    param['property_bag'] = {}
    param['arg_long'] = '--long-argument'

    param.update( kwargs )
    return param

def processScript( script_name, outfile, options ):
    '''process one script.'''

    # call other script
    dirname = os.path.dirname( script_name )
    basename = os.path.basename( script_name)[:-3]

    if options.src_dir: 
        dirname = options.src_dir
        script_name = os.path.join( dirname, basename ) + ".py"

    sys.path.insert(0, dirname )
    module = __import__(basename)

    E.Start = LocalStart
    E.info( "loaded modules %s" % module)
    try:
        module.main( argv = ["--help"])
    except DummyError:
        pass

    # get script's docstring
    docstring = module.__doc__

    # for k in dir(PARSER):
    #     print k, getattr(PARSER, k)
    # for option in PARSER.option_list:
    #     print option, option.type, option.help, option._short_opts, option._long_opts, option.default

    #@prefix clp: <http://www.humgen.nl/climate/ontologies/clp#> .
    #@prefix co: <http://www.isi.edu/ikcap/Wingse/componentOntology.owl#> .
    #@prefix dcterms: <http://purl.org/dc/terms/> .

    # n = Namespace("http://example.org/people/")
    g = Generator()                            

    data = collections.defaultdict( str )

    data['meta_title'] = 'Interface generator for CGAT scripts'
    data['meta_author'] = 'Andreas Heger'
    data['meta_version'] = 0.1

    data['name'] = basename
    data['interpreter'] = 'python'
    data['property_bag'] = {}
    data['description'] = getDescription( basename, docstring )
    data['help'] = docstring
    data['version'] = "1.0"
    data['owner'] = "CGAT"
    data['email'] = "andreas.heger@gmail.com"
    data['binary'] = script_name

    # does not output multiple files 
    data['multiple_output_files'] = False
    
    input_format, output_format = guessFormats( basename, docstring )

    stdin = {}
    stdin['name'] = 'input_file'
    stdin['ns_name'] = 'input_file'
    stdin['type'] = 'stdin'
    stdin['label'] = 'input file'
    stdin['description'] = 'input file'
    stdin['choices'] = None
    stdin['format'] = MAP_TYPE2FORMAT.get( input_format, input_format )
    stdin['rank'] = 1
    stdin['display'] = 'show'
    stdin['min_occurrence'] = 1
    stdin['max_occurrence'] = 1
    stdin['value'] = ""
    stdin['arg'] = "&lt;"
    stdin['arg_long'] = ""
    stdin['property_bag'] = {}
    stdin['dependencies'] = {}

    stdout = {}
    stdout['name'] = 'tsvfile'
    stdout['ns_name'] = 'tsvfile'
    stdout['type'] = 'stdout'
    stdout['label'] = 'table'
    stdout['description'] = 'bam file'
    stdout['choices'] = None
    stdout['format'] = MAP_TYPE2FORMAT.get( output_format, output_format )
    stdout['rank'] = 1
    stdout['display'] = 'show'
    stdout['min_occurrence'] = 1
    stdout['max_occurrence'] = 1
    stdout['value'] = ""
    stdout['arg'] = "&gt;"
    stdout['arg_long'] = ""
    stdout['property_bag'] = {}
    stdout['dependencies'] = {}

    outputs = [ stdout ]

    data['parameters'] = [stdin, stdout ]

    defaults = PARSER.get_default_values()

    # flag to indicate wether script needs to go through cgat_wrapper.py
    use_wrapper = False

    for option in PARSER.option_list:
        # ignore options added by optparse
        if option.dest == None: continue
        
        # ignore benchmarking options
        if option.dest.startswith("timeit"):continue

        # ignore options related to forcing output
        if "force" in option.dest: continue

        # ignore some special options:
        #if option.dest in ("output_filename_pattern", ): 
        #    continue

        # ignore output options
        if option.dest in ("stdin", "stdout", "stdlog", "stderr", "loglevel" ): continue

        # remove default from help string
        option.help = re.sub( "\[[^\]]*%default[^\]]*\]", "", option.help)

        param = buildParam()

        # get command line option call (long/short option)
        try:
            param['arg'] = option._short_opts[0]
        except IndexError: pass

        try:
            param['arg_long'] = option._long_opts[0] 
        except IndexError: pass

        assert 'arg' in param or 'arg_long' in param

        # print "----------------------------------"
        # print [(x,getattr(option,x)) for x in dir( option )]

        param['name'] = option.dest
        param['ns_name'] = option.dest
        if option.type == "int":
            param['type'] = "integer"
        elif option.type == "float":
            param['type'] = "float"
        elif option.type == "string":
            param['type'] = "text"
            if option.metavar:
                mvar = option.metavar.lower()
                if mvar in MAP_TYPE2FORMAT:
                    param['format'] = MAP_TYPE2FORMAT[mvar]
                    param['type'] = "data"
                if mvar == "bam":
                    use_wrapper = True
                    data['parameters'].append( buildParam( name = 'wrapper_bam_file',
                                                           ns_name = 'wrapper_bam_file',
                                                           arg_long = '--wrapper-bam-file',
                                                           label = option.dest,
                                                           type = 'data',
                                                           format = 'bam',
                                                           help = option.help,
                                                           value = getattr( defaults,  option.dest ) ) )

                    data['parameters'].append( buildParam( name = 'wrapper_bam_index',
                                                           ns_name = 'wrapper_bam_index',
                                                           arg_long = '--wrapper-bai-file',
                                                           type = 'data',
                                                           value = '${wrapper_bam_file.metadata.bam_index}',
                                                           display = 'hidden' ) )

                    # use long argument
                    data['parameters'].append( buildParam( name = 'wrapper_bam_option',
                                                           ns_name = 'wrapper_bam_option',
                                                           arg_long = '--wrapper-bam-option',
                                                           value = param['arg_long'],
                                                           display = 'hidden' ) )
                    
                    continue

        elif option.type == "choice":
            param['type'] = "select"
            param['choices'] = option.choices
            if option.action == "append":
                param['multiple'] = True
        elif option.action.startswith("store"):
            param['type'] = "boolean"
        else:
            raise ValueError("unknown type for %s" % str(option))
            
        param['label'] = option.dest
        param['description'] = option.help
        param['rank'] = 1
        param['display'] = 'show'
        param['min_occurrence'] = 0
        param['max_occurrence'] = 1

        # get default value
        param['value'] = getattr( defaults,  option.dest )
            
        param['dependencies'] = {}
        param['property_bag'] = {}

        if option.dest == "genome_file":
            param['property_bag'] = { 'from_loc' : 'path',
                                      'loc_id' : 'sam_fa',
                                      'loc_id_filter' : '1'}


        # deal with multiple output files:
        if option.dest == "output_filename_pattern":
            use_wrapper = True
            data['parameters'].append( buildParam( name = 'wrapper_html_file',
                                                   ns_name = 'wrapper_html_file',
                                                   arg_long = '--wrapper-html-file',
                                                   value = '$html_file',
                                                   display = 'hidden' ) )

            data['parameters'].append( buildParam( name = 'wrapper_html_dir',
                                                   ns_name = 'wrapper_html_dir',
                                                   arg_long = '--wrapper-html-dir',
                                                   value = '$html_file.files_path',
                                                   display = 'hidden' ) )

            outputs.append( buildParam( name = 'html_file',
                                        ns_name = 'html_file',
                                        format= 'html',
                                        label= 'html' ),
                            )
            continue


        data['parameters'].append( param )


    if options.output_format == "rdf":
        outfile.write( g.serialize(data, format='turtle') + "\n" )

    elif options.output_format == "galaxy":

        if use_wrapper:
        
            # add hidden option for wrapper
            param = buildParam( 
                name = 'wrapper-command',
                ns_name = 'wrapper-command',
                display = 'hidden',
                type = 'text',
                value = data['binary'],
                label = 'wrapper',
                description = 'wrapper',
                arg_long = "--wrapper-command" )
            
            data['parameters'].append( param )
        
            # point to wrapper
            data['binary'] = os.path.join( dirname, "cgat_galaxy_wrapper.py" )

        displayMap = collections.defaultdict( list )

        for param in data['parameters']:
            displayMap[ param['display'] ].append( param ) 

        displayMap['normal'] = displayMap['show']
        
        target = Template( open('/ifs/devel/andreas/cgat/scripts/cgat2rdf/galaxy.xml').read() )
        outfile.write( target.render( data = data, 
                                      displayMap = displayMap,
                                      outputs = outputs ) + "\n" )



def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                             usage = globals()["__doc__"] )

    parser.add_option( "-f", "--format", dest="output_format", type="choice",
                       choices = ("rdf", "galaxy"),
                       help = "output format [%default]. ")

    parser.add_option( "-l", "--list", dest="filename_list", type="string",
                       help = "filename with list of files to export [%default]. ")

    parser.add_option( "-s", "--source-dir", dest="src_dir", type="string",
                       help = "directory to look for scripts [%default]. ")

    parser.add_option( "-r", "--input-regex", dest="input_regex", type="string",
                       help = "regular expression to extract script name [%default]. ")

    parser.add_option( "-p", "--output-pattern", dest="output_pattern", type="string",
                       help = "pattern to build output filename. Should contain an '%s' [%default]. ")
    
    parser.set_defaults( output_format = "rdf",
                         src_dir = None,
                         input_regex = None,
                         output_pattern = None,
                         filename_list = None )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) == 0:
        E.info( "reading script names from stdin" )
        for line in options.stdin:
            if line.startswith("#"): continue
            args.append( line[:-1].split("\t")[0] )

    # start script in order to build the command line parser
    global ORIGINAL_START
    ORIGINAL_START = E.Start

    if options.output_pattern and not options.input_regex:
        raise ValueError("please specify --input-regex when using --output-pattern")

    if options.output_format == "galaxy":
        options.stdout.write( '''<section name="CGAT Tools" id="cgat_tools">\n''')

    for script_name in args:
        if not script_name.endswith(".py"):
            raise ValueError( "expected a python script ending in '.py'" )

        if options.input_regex:
            try:
                input_string = re.search( options.input_regex, script_name ).groups()[0]
            except AttributeError:
                E.warn("can not parse %s - skipped", script_name )
                continue

        if options.output_pattern:
            outfile_name = re.sub( "%s", input_string, options.output_pattern )
            outfile = open( outfile_name, "w" )
        else:
            outfile = options.stdout

        E.info( "input=%s, output=%s" % (script_name, outfile_name ))
        processScript( script_name, outfile, options )
        
        if options.output_format == "galaxy":
            options.stdout.write( '''   <tool file="cgat/%s" />\n''' % outfile_name )

        if outfile != options.stdout:
            outfile.close()

    if options.output_format == "galaxy":
        options.stdout.write( '''</section>\n''')

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   Gene prediction pipeline 
#
#   $Id: SaryFasta.py 2784 2009-09-10 11:41:14Z andreas $
#
#   Copyright (C) 2007 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#################################################################################
"""
SaryFasta.py - index fasta files by suffix array
===================================================

Subroutines for working on I/O of large genomic files.

Index a fasta file to retrieve sequences by suffix-array fragment
search.

python SaryFasta.py [options] name [ files ]
"""

import os, sys, array, string, re, types, optparse, time, struct, hashlib, base64, shutil, subprocess
import glob, random

##------------------------------------------------------------
def getHID ( sequence ):
    """returns a hash identifier for a sequence.
    """
    
    # do the encryption
    h = hashlib.md5(sequence).digest()
    
    # map to printable letters: hid has length 22, so the padded '=' are
    # truncated. You have to add them, if you ever want to decode,
    # but who would do such a thing :=)

    r = base64.encodestring(h)[0:22]
    
    # finally substitute some characters:
    # '/' for '_', so we have legal file names
    # '[' for '+' and ']' for '=' for internet-applications
    
    hid = string.replace(r  , '/', '_')
    hid = string.replace(hid, '+', '[')
    hid = string.replace(hid, '=', ']')
                                                
    return hid

##------------------------------------------------------------
def createDatabase( db, filenames,
                    buf_size=400000000,
                    force = False,
                    regex_identifier = None):
    """index files in filenames to create database.

    buf_size: buffer size for a sary chunk.

    Two new files are created - db.fasta and db_name.idx
    
    regex_identifier: pattern to extract identifier from description line.
    If None, the part until the first white-space character is used.
    """

    index_name = db + ".idx"
    db_name = db + ".dat"
    if db in filenames:
        raise ValueError( "database (%s) is part of input set." % db)

    if os.path.exists( db_name ) and not force:
        raise ValueError( "database %s already exists." % db)

    if os.path.exists( index_name ) and not force:
        raise ValueError( "database index %s already exists." % index_name )
    
    outfile_index = open( index_name, "w" )

    os.mkdir( db_name )
    files_to_index = []
    fn = db_name + "/part%i" % len(files_to_index)
    outfile_src = open( fn, "w" )
    files_to_index.append(fn)
    
    if type(filenames) == types.StringType:
        filenames = [filenames]

    identifiers = {}
    lsequence = 0
    identifier_pos, sequence_pos = 0, 0
    noutput = 0
    
    for filename in filenames:

        if filename[:-3] == ".gz":
            infile = gzip.open(filename, "r")
        else:
            infile = open( filename, "r")

        fragments = []
        first = True
        
        for line in infile:

            if line[0] == "#":  continue
            
            if line[0] == ">" :
                
                if not first:
                    
                    f = "".join(fragments)
                    
                    if len(f) + noutput > buf_size:
                        outfile_src.close()
                        fn = db_name + "/part%i" % len(files_to_index)                        
                        outfile_src = open( fn, "w" )
                        files_to_index.append(fn)
                        noutput = 0

                    outfile_src.write( f + "\n" )
                    outfile_index.write("%s\t%s\n" % (getHID(f), identifier) )

                    noutput += len(f)
                    fragments = []

                first = False
                
                if regex_identifier:
                    try:
                        identifier = re.search(regex_identifier, line[1:-1]).groups()[0]
                    except AttributeError:
                        raise "could not parse identifer from line %s" % line[1:-1]
                else:
                    identifier = re.split("\s", line[1:-1])[0]
                    
                ## check for duplicate identifiers
                if identifier in identifiers:
                    raise ValueError, "%s occurs more than once in %s and %s: line=%s" %\
                          (identifier, identifiers[identifier], filename, line[1:-1])
                identifiers[identifier] = filename
                
            else:
                s = re.sub( "\s", "", line.strip() )
                fragments.append( s )
        
        f = "".join(fragments)
        outfile_src.write( f + "\n" )
        outfile_index.write("%s\t%s\n" % (getHID(f), identifier) )

        infile.close()
                    
    ## build indicies
    for filename in files_to_index:
        
        try:
            retcode = subprocess.Popen( "mksary --quiet %s" % (filename), shell=True)
            
            if retcode < 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode
            else:
                print >>sys.stderr, "Child returned", retcode

        except OSError, e:
            print >>sys.stderr, "Execution failed:", e

class SaryFasta:

    def __init__( self, dbname ):

        self.mNameIndex = dbname + ".idx"
        if not os.path.exists( self.mNameIndex ):
            raise "index file %s does not exist." % (self.mNameIndex)

        self.mNameData = dbname + ".dat"
        if not os.path.exists( self.mNameData ):
            raise "data dir %s does not exist." % (self.mNameData)
        
        self.mDbname = dbname
        self.mIsLoaded = False
        self.mFilenames = map( lambda x: x[:-4], glob.glob( self.mNameData + "/part*.ary"))
        
        if len(self.mFilenames) == 0:
            raise "empty database."

    def __getitem__(self, pattern ):
        """return full length sequence."""
        return self.search( pattern )
        
    def __loadIndex( self ):
        """load complete index into memory."""

        self.mIndex = {}

        for line in open(self.mNameIndex, "r"):

            hid, identifier = line[:-1].split("\t")
            
            self.mIndex[hid] = identifier

        self.mIsLoaded = True
            
    def getDatabaseName( self ):
        """returns the name of the database."""
        return self.mDbname
    
    def search( self, pattern ):
        """search indices with a pattern.
        """

        if not self.mIsLoaded: self.__loadIndex()
        
        result = []
        for filename in self.mFilenames:

            cmd = "sary '%s' %s" % (pattern, filename)
            for line in subprocess.Popen( cmd, shell=True, stdout=subprocess.PIPE).stdout:
                hid = getHID( line[:-1] )
                if hid not in self.mIndex:
                    raise "inconsistency: can not find identifier for hid %s in index.\n" % (hid)
                
                result.append( self.mIndex[hid] )

        return result

## Test function for benchmarking purposes
def benchmarkRandomFragment( fasta, size ):
    """returns a random fragment of size."""

    contig, strand, start, end = fasta.getRandomCoordinates( size )
    s = fasta.getSequence( contig, strand, start, end )
    return s

def verify( reference, fasta, 
            num_iterations, fragment_size,
            stdout = sys.stdout, quiet = False ):
    """verify two databases.

    Get segment from fasta and check for presence in fasta2.
    """
    if not quiet:
        options.stdout.write("verifying %s with %s using %i random segments per sequence of length %i\n" %\
                             (fasta.getDatabaseName(),
                              reference,
                              num_iterations,
                              fragment_size ))
        options.stdout.flush()
    nerrors = 0
    f = []
    for line in open(reference,"r"):
        if line[0] == ">":
            for x in range(num_iterations):
                s = "".join(f)
                if len(s) > fragment_size:
                    c = random.randint( 0, len(s) - fragment_size)
                    pattern = s[c:c+fragment_size]
                    found = set(fasta.search( pattern ))
                    if id not in found:
                        nerrors += 1
                        if not quiet:
                            options.stdout.write("can't find pattern '%s' for %s in %s\n" %\
                                                     (pattern, id, str(found)))
            f = []
            id = re.search(">(\S+)", line[:-1]).groups()[0]
        else:
            f.append( re.sub("\s", "", line[:-1]) )
    return nerrors

if __name__ == "__main__":

    import Experiment

    parser = E.OptionParser( version = "%prog version: $Id: SaryFasta.py 2784 2009-09-10 11:41:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option( "-e", "--extract", dest="extract", type="string",
                       help="extract region ( for testing purposes. Format is contig:strand:from:to." )

    parser.add_option( "-c", "--compression", dest="compression", type="choice",
                       choices=("lzo", "zlib", "gzip", "dictzip", "bzip2", "debug"),
                       help="compress database." )

    parser.add_option( "--random-access-points", dest="random_access_points", type="int",
                       help="save random access points every # number of nucleotides." )
    

    parser.add_option( "-f", "--input-format", dest="input_format", type="choice",
                       choices=("one-forward-open", "zero-both-open" ),
                       help="coordinate format of input." )

    parser.add_option( "-s", "--synonyms", dest="synonyms", type="string",
                       help="list of synonyms, comma separated with =, for example, chr1=chr1b" )

    parser.add_option( "-b", "--benchmark", dest="benchmark", action="store_true",
                       help="benchmark read access." )
    
    parser.add_option( "--benchmark-num-iterations", dest="benchmark_num_iterations", type="int",
                       help="number of iterations for benchmark [%DEFAULT%]." )

    parser.add_option( "--benchmark-fragment-size", dest="benchmark_fragment_size", type="int",
                       help="benchmark: fragment size [%DEFAULT%]." )

    parser.add_option( "--verify", dest="verify", type="string",
                       help="verify against other database.")

    parser.add_option( "-a", "--clean-sequence", dest="clean_sequence", action="store_true",
                       help="remove X/x from DNA sequences - they cause errors in exonerate." )

    parser.add_option( "--regex-identifier", dest="regex_identifier", type="string",
                       help="regular expression for extracting the identifier from fasta description line." )

    parser.set_defaults(
        extract = None,
        input_format = "zero-both-open",
        benchmark_fragment_size = 1000,
        benchmark_num_iterations = 1000000,
        benchmark = False,
        compression = None,
        random_access_points = 0,
        synonyms = None,
        verify = None,
        verify_num_iterations = 1,
        verify_fragment_size = 20,
        clean_sequence = False,
        regex_identifier = None)
    
    (options, args) = Experiment.Start( parser )

    if options.extract:
        fasta = SaryFasta( args[0] )
        converter = getConverter( options.input_format )
        
        contig, strand, start, end = options.extract.split(":")
        start, end = map( int, (start, end) )
        sequence = fasta.getSequence( contig, strand,
                                      start, end,
                                      converter = converter )
        options.stdout.write( ">%s\n%s\n" % \
                              ( options.extract, sequence ) )
    elif options.benchmark:
        import timeit
        timer = timeit.Timer( stmt="benchmarkRandomFragment( fasta = fasta, size = %i)" % (options.benchmark_fragment_size),
                              setup="""from __main__ import benchmarkRandomFragment,SaryFasta\nfasta=SaryFasta( "%s" )""" % (args[0] ) )

        t = timer.timeit( number = options.benchmark_num_iterations )
        options.stdout.write("iter\tsize\ttime\n" )
        options.stdout.write("%i\t%i\t%i\n" % (options.benchmark_num_iterations, options.benchmark_fragment_size, t ) )
    elif options.verify:
        fasta = SaryFasta( args[0] ) 
        nerrors1 = verify( options.verify, 
                           fasta,
                           options.verify_num_iterations,
                           options.verify_fragment_size,
                           stdout=options.stdout )
        options.stdout.write("errors=%i\n" % (nerrors1) )        
    else:
        if options.loglevel >= 1:
            options.stdlog.write("# creating database %s\n" % args[0])            
            options.stdlog.write("# indexing the following files: \n# %s\n" %\
                                 (" \n# ".join( args[1:] ) ))
            options.stdlog.flush()

        if len(args) < 2:
            print USAGE
            sys.exit(1)
            
        createDatabase( args[0], args[1:], 
                        regex_identifier = options.regex_identifier )
    
    Experiment.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
IndexedGenome.py - Wrappers for several interval indices
=========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import NCL as ncl
from bx.intervals.intersection import Intersecter, Interval

class IndexedGenome:
    '''Genome with indexed intervals.
    '''

    index_factory = ncl.NCL
    def __init__( self ): 
        self.mIndex = {}

    def add( self, contig, start, end, value ):
        
        if contig not in self.mIndex: 
            self.mIndex[contig] = self.index_factory()
        self.mIndex[contig].add( start,end,value )

    def __getitem__(self, args):
        '''return intervals overlapping with key.'''
        if args[0] not in self.mIndex: 
            raise KeyError("contig %s not in index" % args[0] )
        
        return self.mIndex[args[0]].find( args[1],args[2] )

    def contains( self, contig, start, end ):
        if contig not in self.mIndex: return False
        return len(list(self.mIndex[contig].find( start, end))) > 0

    def get(self, contig, start, end):
        '''return intervals overlapping with key.'''
        if contig not in self.mIndex: 
            raise KeyError("contig %s not in index" % contig )
        
        return self.mIndex[contig].find( start, end )

    def __len__(self):
        '''return number of contigs.'''
        return len(self.mIndex)

class Simple( IndexedGenome ):
    '''index intervals without storing a value.'''
    index_factory = ncl.NCLSimple
    
    def __init__(self, *args, **kwargs ):
        IndexedGenome.__init__(self, *args, **kwargs )

    def add( self, contig, start, end ):
        
        if contig not in self.mIndex: 
            self.mIndex[contig] = self.index_factory()
        self.mIndex[contig].add( start,end )


class Quicksect( IndexedGenome ):
    '''index intervals using quicksect. 
    
    Permits finding closest interval in case there is 
    no overlap.
    '''
    index_factory = Intersecter
    
    def __init__(self, *args, **kwargs ):
        IndexedGenome.__init__(self, *args, **kwargs )

    def add( self, contig, start, end, value ):
        
        if contig not in self.mIndex: 
            self.mIndex[contig] = self.index_factory()
        self.mIndex[contig].add_interval( Interval( start,end, value ) )
        
    def get(self, contig, start, end ):
        '''return intervals overlapping with key.'''
        if contig not in self.mIndex: 
            raise KeyError("contig %s not in index" % contig )
        
        return [(x.start, x.end, x.value) for x in self.mIndex[contig].find( start, end ) ]
    
    def before( self, contig, start, end, num_intervals = 1, max_dist = 2500 ):
        '''get closest interval before *start*.''' 
        if contig not in self.mIndex: 
            raise KeyError("contig %s not in index" % contig )
        return [(x.start, x.end, x.value) for x in self.mIndex[contig].before_interval( Interval( start,end), 
                                                                                        num_intervals = 1,
                                                                                        max_dist = max_dist ) ]


    def after( self, contig, start, end, num_intervals = 1, max_dist = 2500 ):
        '''get closest interval after *end*.''' 
        if contig not in self.mIndex: 
            raise KeyError("contig %s not in index" % contig )
        return [(x.start, x.end, x.value) for x in self.mIndex[contig].after_interval( Interval( start,end), 
                                                                                       num_intervals = 1,
                                                                                       max_dist = max_dist ) ]





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperBlastZ.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, optparse, subprocess

"""Wrapper for BlastZ.
"""

import Genomics
import Experiment as E

class BlastZError(Exception):
    pass

class BlastZ:

    # options for sensitive alignment of short (<10Mb) sequences
    # according to blastz manual.
    mOptions = "C=2 B=0 T=0 W=6 K=2200"
    mExecutable = "blastz"
    mStderr = sys.stderr
    
    def __init__( self, options=None):

        if options:
            self.mOptions = options

    def CreateTemporaryFiles( self ):
        """create temporary files."""
        self.mTempDirectory = tempfile.mkdtemp()
        
        self.mFilenameTempSeq1 = self.mTempDirectory + "/seq1"
        self.mFilenameTempSeq2 = self.mTempDirectory + "/seq2"

    def DeleteTemporaryFiles( self ):
        """clean up."""
        os.remove( self.mFilenameTempSeq1 )
        os.remove( self.mFilenameTempSeq2 )
        os.rmdir( self.mTempDirectory )

    def SetStderr( self, file = None):
        """set file for dumping stderr."""
        self.mStderr= file
        
    def ParseResult( self, out, result ):
        """parse BLASTZ output.

        TODO: check for subopt alignments - does blastz do these?
        """
        result.clear()

        keep = False

        self.mReverseComplement = False

        def block_parser( out ):
            keep = False
            lines = []
            for line in out.split("\n"):
                if not line: continue

                if line[0] == "}":
                    yield id, lines
                    
                if re.match( "[shaxmd] {", line):
                    id = line[0]
                    keep = True
                    lines = []
                    continue
                
                if keep:
                    lines.append( line.strip() )
                    
        for id, lines in block_parser(out):

            if id == "a":
                for line in lines:
                    d = line.split(" ")
                    if d[0] == "l":
                        row_from, col_from, row_to, col_to = map(int, d[1:5])
                        result.addDiagonal( row_from-1, row_to, col_from - row_from )
            
            elif id == "s":
                d = lines[1].split(" ")
                self.mReverseComplement = d[3] == "1"

    def isReverseComplement( self ):
        return self.mReverseComplement

    def Align( self, seq1, seq2, result ):
        """align two sequences with BlastZ."""
        
        self.CreateTemporaryFiles()

        f = open(self.mFilenameTempSeq1, "w")
        f.write(">seq1\n%s" % seq1 )
        f.close()

        f = open(self.mFilenameTempSeq2, "w")
        f.write(">seq2\n%s" % seq2 )
        f.close()
        
        statement = string.join( ( self.mExecutable,
                                   self.mFilenameTempSeq1,
                                   self.mFilenameTempSeq2,                                   
                                   self.mOptions),
                                 " ")

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempDirectory,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise BlastZError, "Error in running BLASTZ \n%s" % err

        self.ParseResult( out, result )

        self.mStderr.write( err )

        self.DeleteTemporaryFiles()
     
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: WrapperBlastZ.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-i", "--input-file-seq1", dest="input_filename_seq1", type="string",
                      help="input filename for sequence 1.",
                      metavar="FILE" )

    parser.add_option("-j", "--input-file-seq2", dest="input_filename_seq2", type="string",
                      help="input filename for sequence 2.",
                      metavar="FILE" )

    parser.add_option("-o", "--options", dest="options", type="string",
                      help="BlastZ options." )

    parser.set_defaults( \
        input_filename_seq1 = None,
        input_filename_seq2 = None,
        options = "B=0 C=2")
    
    (options, args) = E.Start( parser ) 
    
    wrapper = BlastZ( options.options )

    import alignlib
    seqs1 = Genomics.ReadPeptideSequences( open(options.input_filename_seq1, "r") )
    seqs2 = Genomics.ReadPeptideSequences( open(options.input_filename_seq2, "r") )
    seq1 = seqs1[seqs1.keys()[0]]
    seq2 = seqs2[seqs2.keys()[0]]    
    result = alignlib.makeAlignmentVector()
    wrapper.Align( seq1, seq2, result) 

    print str( alignlib.AlignmentFormatExplicit( result,
                                                 alignlib.makeSequence( seq1 ),
                                                 alignlib.makeSequence( seq2 ) ) )
    
    E.Stop()
        
            
        
                                 
        
        
        
        
    
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
join_tables.py - join tables
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python join_tables.py --help

Type::

   python join_tables.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

""" program $Id: join_tables.py 2782 2009-09-10 11:40:29Z andreas $

"""
import CGAT.Experiment as E

USAGE = """python %s < stdin > stdout

OPTIONS:

-t, --titles=           column titles
-m, --missing=          missing value
-h, --headers=          add headers for files
-s, --sort=             sort by column titles (given by sort order)
'#' at start of line is a comment
""" % sys.argv[0]

def GetData( f ):

    while 1:
        line = f.readline()
        if not line: return None
        if line[0] == "#": continue
        break
    
    return line[:-1].split("\t")
        
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: join_tables.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to do join on. Files have to sorted alphabeticlly and incrementally on these columns." )
    parser.add_option("-m", "--missing", dest="missing_value", type="string",
                      help="value for missing entries." )

    parser.set_defaults(
        headers = False,
        pattern_filename = None,
        title = "",
        footer = "",
        columns = "1",
        missing_value = "",
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if len(args) < 2:
        raise "there have to be at least two tables."

    if options.columns: options.columns = map(lambda x: int(x) - 1, options.columns.split(","))
    
    ## open all files
    files = []
    for filename in args:

        if os.path.exists( filename ):
            files.append( open(filename, "r") )

    if len(files) <= 1:
        raise "less than two files opened."

    nfiles = len(files)
    
    ## go through all files and iteratively find smallest entry
    data = []
    entries = []
    lengths = []
    takes = []
    
    for f in files:
        d = GetData( f )
        l = len(d)
        t = []
        for x in range( l ):
            if x not in options.columns:
                t.append(x)
                
        takes.append(t)
        data.append( map( lambda x: d[x], t) )
        entries.append( map( lambda x: d[x], options.columns) )
        lengths.append( len(t) )

    activa = list(range( nfiles ))
    ## take only first entry for duplicate entries
    ## columns need to be sorted incrementally on all columns
    while len(activa) > 0:

        line = []
        
        min_field = min([entries[x] for x in activa])
        
        for f in range(nfiles):

            if files[f] and entries[f] == min_field:
                
                line += data[f]

                while entries[f] == min_field:
                    
                    d = GetData(files[f])

                    if not d:
                        files[f].close()
                        files[f] = None
                        activa.remove( f )
                        break
                    
                    data[f] = map( lambda x: d[x], takes[f])
                    entries[f] = map( lambda x: d[x], options.columns)
                
            else:
                line += [options.missing_value for x in range(lengths[f])]
                
        options.stdout.write( "\t".join(min_field) + "\t" + "\t".join(line) + "\n" )

    ## close all files
    for f in files:
        if f:
            f.close()

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
ProfileLibraryCompass.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse, time, math, shutil

#--------------------------------------------------------
#--------------------------------------------------------
#--------------------------------------------------------
# usage message
#--------------------------------------------------------
USAGE="""python %s [OPTIONS]

work with library of compass profiles.

Actions include: 
   * create: create a profile library
   * verify: verify a profile library
   * merge:  merge several profile libraries
   * split:  split a profile library into smaller sections

This is version $Id: ProfileLibraryCompass.py 2781 2009-09-10 11:33:14Z andreas $.
""" % sys.argv[0]

#--------------------------------------------------------
#--------------------------------------------------------
#--------------------------------------------------------
# import of user libraries
#--------------------------------------------------------
import Experiment
import Mali
from ProfileLibrary import ProfileLibrary

class CompassProfile:

    def __init__(self, profile ):
        if profile[0] != "#":
            raise "malformatted profile."
        
        pos = profile.find("\n")
        self.mName = profile[2:pos]
        self.mProfile = profile[pos+1:]
            
    def save( self, outfile ):
        outfile.write( "# %s\n" % self.mName )
        outfile.write( self.mProfile )
        
    def setName( self, name ):
        self.mName = name

    def getName( self ):
        return self.mName

class ProfileLibraryCompass( ProfileLibrary ):

    mIndexer = "mk_compass_db"
    
    mSuffixDatabase = ".cdb"
    mSuffixIndex = ".cix"

    def __init__(self, name, mode = "r" ):

        ProfileLibrary.__init__( self, name, mode )

        self.mMaxFilesPerChunk = 2000

    def getProfile( self, name ):
        """get a profile from this library.

        This function returns a compass profile as a string.
        """

        if name not in self.mIndex: raise KeyError

        first_pos, last_pos = self.mIndex[name]
        self.mInfileDatabase.seek( first_pos )
        return CompassProfile(self.mInfileDatabase.read( last_pos - first_pos ) )

    def __create( self, tempdir, nchunk ):
        """create chunk of a profile library from file.
        This is necessary due to a 2Gb file size limit in the compass
        executable that I downloaded.
        """
        
        # run mk_compass_db
        fn = "%s_%i" % (os.path.abspath(self.mFilenameProfiles ), nchunk)
        statement = "%s -i files.list -o %s" % (self.mIndexer, fn )
        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = tempdir,
                              close_fds = True)                              

        (out, err) = s.communicate()
        
        if s.returncode != 0:
            raise "Error in running %s \n%s\n%s\nTemporary directory in %s" % (statement, err, out, tempdir)

        ## clean up
        shutil.rmtree( tempdir )
        
        return fn

    def create( self, infile ):
        """create profile library from file."""
        
        ninput, noutput = 0, 0

        outfile_components = None

        # write multiple alignment as individual files
        nchunk = 0
        chunks = []

        while mali.readFromFile( infile, format="profile" ):

            ninput += 1
            
            if not outfile_components:
                tempdir = tempfile.mkdtemp()
                outfile_components = open( tempdir + "/files.list", "w" )
                
            fn = tempdir + "/%s" % mali.getName()
            outfile = open( fn, "w" )
            mali.writeToFile( outfile, format="plain-fasta" )
            outfile.close()
            outfile_components.write( "%s\n" % mali.getName() )
            noutput += 1            

            if ninput % self.mMaxFilesPerChunk == 0:
                outfile_components.close()
                if self.mLogLevel >= 2:
                    self.mStdLog.write("# building profiles for chunk %i at %i\n" % (nchunk, ninput) )
                    self.mStdLog.flush()

                chunks.append( self.__create( tempdir, nchunk ) )
                nchunk += 1
                outfile_components = None

        outfile_components.close()
        chunks.append( self.__create( tempdir, nchunk ) )
        
        # combine into a single file and remove temporary files
        if self.mLogLevel >= 2:
            self.mStdLog.write("# concatenating %i chunks\n" % (len(chunks) ))
            self.mStdLog.flush()

        outfile = open( os.path.abspath( self.mFilenameProfiles), "w" )
        for filename in chunks:
            infile = open(filename, "r" )
            for line in infile:
                outfile.write( line )
            infile.close()
            os.remove( filename )
        outfile.close()
        
        # index the profile database
        if self.mLogLevel >= 2:
            self.mStdLog.write("# indexing the database\n" )
            self.mStdLog.flush()

        self.index()

        return ninput, noutput

    def index( self ):
        """build index for database."""

        outfile_index = open( self.mFilenameIndex, "w" )
        infile_database = open( self.mFilenameProfiles, "r" )
        
        pos = infile_database.tell()
        last_name = None
        skip = False
        while 1:

            pos = infile_database.tell()
            line = infile_database.readline()
            if not line: break

            if line[0] == "#": 
                if skip: 
                    skip = False
                    continue
                
                if last_name:
                    outfile_index.write( "%s\t%i\t%i\n" % (last_name, first_pos, pos ) )
                last_name = line[2:-1]
                skip = True
                first_pos = pos

        if last_name:
            pos = infile_database.tell()
            outfile_index.write( "%s\t%i\t%i\n" % (last_name, first_pos, pos ) )
            
        outfile_index.close()
        infile_database.close()
        
    def verify( self, infile ):
        """verify data in database against original data."""

        if not self.mIndex: self.__loadIndex()
        
        ninput, nfound, nnotfound, ndifferent = 0,0,0,0
        while mali.readFromFile( sys.stdin, format="profile" ):

            ninput += 1
            
            raise "not implemented"
            
            p2 = self.getProfile( mali.getName() )

            if p1.getLength() != p2.getLength() or \
                    str(p1) != str(p2):
                ndifferent += 1
                continue
            
            nfound += 1

        return ninput, nfound, nnotfound, ndifferent

#--------------------------------------------------------
#--------------------------------------------------------
#--------------------------------------------------------
# main part of script
#--------------------------------------------------------
if __name__ == "__main__":

    #--------------------------------------------------------
    # command line parsing options
    parser = E.OptionParser( version = "%prog version: $Id: ProfileLibraryCompass.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-p", "--prefix", dest="prefix", type="string",
                      help="prefix to use for the profile library." )

    parser.add_option("-a", "--action", dest="action", type="choice",
                      choices=("create", "verify", "merge", "split", "stats", "index" ),
                      help="action to undertake." )

    parser.set_defaults( prefix = "profiles", 
                         action = "create",
                         weightor = None,
                         )

    (options, args) = Experiment.Start( parser )

    #--------------------------------------------------------
    # main part of script
    mali = Mali.Mali()

    if options.action in ("create", "merge" ):
        mode = "w"
    else:
        mode = "r"

    plib = ProfileLibraryCompass( options.prefix, mode )
    plib.setOptions( options )

    if options.action == "verify":
        ninput, nfound, nnotfound, ndifferent = plib.verify(sys.stdin)
        if options.loglevel >= 1:
            options.stdlog.write( "# verify: ninput=%i, nfound=%i, nnotfound=%i, ndifferent=%i\n" % (ninput, nfound, nnotfound, ndifferent))

    elif options.action == "create":
        # create a new profile library
        ninput, noutput = plib.create( sys.stdin )
        if options.loglevel >= 1:
            options.stdlog.write( "# ninput=%i, noutput=%i\n" % (ninput, noutput) )

    elif options.action == "index":
        plib.index()

    elif options.action == "merge":
        for library in args:
            if options.loglevel >= 1:
                options.stdlog.write("# adding library %s\n" % library )
                options.stdlog.flush()

            other_lib = ProfileLibraryCompass( library )

            for name, profile in other_lib.iteritems():
                plib.appendProfile( name, profile )

    elif options.action == "stats":
        options.stdout.write("profiles\t%i\n" % len(plib) )
        
    #--------------------------------------------------------
    # general cleaning up
    Experiment.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2peakshape.py - compute peak shape features from a bam-file
==============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes a :term:`bed` formatted file with regions of interest,
for example binding intervals from a ChIP-Seq experiment. Using a collection
of aligned reads is a :term:`bam` formatted file, the script outputs a collection
of features describing the peak shape.

This script is designed with a slight emphasis on ChIP-Seq datasets. For example,
for the purpose of visualizing binding profiles of transcription factors
ChIP-Seq purely based on the peaks regions defined by MACS, and without the need to use any genomic
annotations (e.g. ENSEMBL, refseq). The reason is that, bam2peakshape.py is able to
center the counting window at the summit of every individual peak. bam2peakshape.py is also able
to: (1) plot the control RNA-Seq library to enable side-by-side comparison of treatment vs control;
(2) randomly shift the set of input resions to generate a background set of regions in order to
provide a peaks profile that can be used as the control.

Alternatively,  you may also consider using :doc:`bam2geneprofile`, which is
designed with a slight emphasis on RNA-Seq datasets, which takes care of spliced reads,
by using the CIGAR string in the BAM file to accurately define the covered bases (when
the --base-accurate-off is not specified, currently it is not specified by default).

The script outputs a tab-separated table on stdout containing features for each interval.
A peak is defined as the location of the highest density in an interval. The width of 
the peak (peak_width) is defined as the region around the peak in which the density does
not drop below a threshold of peak_heigt * 90%.

Among the features output are:

+---------------------+----------------------------------------------------------+
|*Column*             |*Content*                                                 |
+---------------------+----------------------------------------------------------+
|peak_height          |number of reads at peak                                   |
+---------------------+----------------------------------------------------------+
|peak_median          |median coverage compared to peak height                   | 
+---------------------+----------------------------------------------------------+
|interval_width       |width of interval                                         |
+---------------------+----------------------------------------------------------+
|peak_width           |width of peak                                             |
+---------------------+----------------------------------------------------------+
|bins                 |bins for a histogram of densities within the interval.    |
+---------------------+----------------------------------------------------------+
|npeaks               |number of density peaks in interval.                      |
+---------------------+----------------------------------------------------------+
|peak_center          |point of highest density in interval                      |
+---------------------+----------------------------------------------------------+
|peak_relative_pos    |point of highest density in interval coordinates          |
+---------------------+----------------------------------------------------------+
|counts               |counts for a histogram of densities within the interval   |   
+---------------------+----------------------------------------------------------+
|furthest_half_height |Distance of peak center to furthest half-height position  |
+---------------------+----------------------------------------------------------+
|closest_half_height  |Distance of peak center to closest half-height position   |
+---------------------+----------------------------------------------------------+

Additionally, the script outputs a set of matrixes with densities over intervals that
can be used for plotting. The default filenames are ``(matrix|control)_<sortorder>.tsv.gz``,
The names can be controlled with the ``--output-filename-pattern`` option.

.. todo::
   
   paired-endedness is not fully implemented.

Usage
-----

Example::

   python bam2peakshape.py bamfile bedfile

Type::

   python bam2peakshape.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import collections
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam
import CGAT.Bed as Bed
import numpy
import bx.bbi.bigwig_file

try:
    import pyximport
    pyximport.install(build_in_temp=False)
    import _bam2peakshape
except ImportError:
    import CGAT._bam2peakshape as _bam2peakshape

def buildOptionParser( argv ):
    
    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-f", "--format", dest="format", type = "choice",
                       choices = ("bam", "bigwig" ),
                       help = "format of genomic input files for densities "
                              "[%default]" )

    parser.add_option( "-o", "--only-interval", dest="only_interval", action="store_true",
                       help = "only count tags strictly in interval. Otherwise, use window around peak center "
                              " [%default]" )

    parser.add_option( "-w", "--window-size", dest="window_size", type = "int",
                       help = "window size to use"
                              "[%default]" )

    parser.add_option( "-b", "--bin-size", dest="bin_size", type = "int",
                       help = "bin-size for histogram of read depth. "
                              "[%default]" )

    parser.add_option( "-s", "--sort", dest="sort", type = "choice", action = "append",
                       choices = ("peak-height", "peak-width", "unsorted", 
                                  "interval-width", "interval-score" ),
                       help = "output sort order for matrices. "
                              "[%default]" )

    parser.add_option( "-c", "--control-file", dest="control_file", type = "string",
                       help = "control file. If given, two peakshapes are computed, "
                       "one for the primary data and one for the control data. "
                       "The control file is centered around the same "
                       "base as the primary file and output in the same "
                       "sort order as the primary profile to all side-by-side. "
                       "comparisons. "
                       "[%default]" )

    parser.add_option( "-r", "--random-shift", dest="random_shift", action="store_true",
                       help = "shift intervals in random direction up/downstream of interval "
                              "[%default]" )
 
    parser.add_option( "-e", "--centring-method", dest="centring_method", type = "choice",
                       choices = ("reads", "middle"),
                       help = "centring method. Available are: "
                              "reads=use density to determine peak, "
                              "middle=use middle of interval " 
                              "[%default]" )
                              
    parser.add_option( "-n", "--normalization", dest="normalization", type = "choice",
                       choices = ("none", "sum" ),
                       help = "normalisation to perform. "
                              "[%default]" )                                 

    parser.add_option( "--use-strand", dest="strand_specific", action="store_true",
                       help = "use strand information in intervals. Intervals on the "
                       "negative strand are flipped "
                       "[%default]" )
 
    parser.add_option( "-i", "--shift", dest="shift", type = "int",
                       help = "shift for reads. When processing bam files, "
                       "reads will be shifted upstream/downstream by this amount. "
                       "[%default]" )

    parser.set_defaults(
        bin_size = 10,
        shift = 0,
        window_size = 1000,
        sort = [],
        centring_method = "reads",
        control_file = None,
        random_shift = False,
        strand_specific = False,
        format = "bam",
        report_step = 100,
        )

    return parser

def outputResults( result, bins, options ):
    '''ouput results from density profiles.'''

    # center bins
    out_bins = bins[:-1] + options.bin_size

    def writeMatrix( result, sort ):

        outfile_matrix = E.openOutputFile( "matrix_%s.gz" % re.sub("-", "_", sort ) )            
        outfile_matrix.write( "name\t%s\n" % "\t".join( map(str, out_bins )))
        
        if result[0][2] != None:
            outfile_control = E.openOutputFile( "control_%s.gz" % re.sub("-", "_", sort ) )            
            outfile_control.write( "name\t%s\n" % "\t".join( map(str, out_bins )))

        if result[0][3] != None:
            outfile_shift = E.openOutputFile( "shift_%s.gz" % re.sub("-", "_", sort ) )            
            outfile_shift.write( "name\t%s\n" % "\t".join( map(str, out_bins )))
            
        n = 0
        for features, bed, control, shifted in result:
            n += 1
            if "name" in bed: name = bed.name
            else:name = str(n)
            bins, counts = features[-2], features[-1]
            outfile_matrix.write( "%s\t%s\n" % (name, "\t".join(map(str,counts))) )
            if control:
                outfile_control.write( "%s\t%s\n" % (name, "\t".join(map(str,control.counts))) )
            if shifted:
                outfile_shift.write( "%s\t%s\n" % (name, "\t".join(map(str,shifted.counts))) )

        outfile_matrix.close()

    n = 0
    for features, bed, control, shifted in result:
        n += 1
        if "name" in bed: name = bed.name
        else: name = str(n)
        options.stdout.write( "%s\t%i\t%i\t%s\t" % (bed.contig, bed.start, bed.end, name ) )

        options.stdout.write( "\t".join( map(str, features[:-2] ) ) )
        bins, counts = features[-2], features[-1]
        options.stdout.write( "\t%s" % ",".join( map(str, bins )))
        options.stdout.write( "\t%s" % ",".join( map(str, counts)))
        options.stdout.write( "\n" )

    norm_result = []
    if options.normalization == "sum":
        E.info("Starting sum normalization")
        # get total counts across all intervals
        norm = 0.0
        for features, bed, control, shifted in result:
            counts = features[-1]
            norm += sum(counts)
        norm /= 1000000
        E.info("norm = %i" % norm)
        
        # normalise
        for features, bed, control, shifted in result:
            counts = features[-1]
            norm_counts = []
            for c in counts:
                norm_counts.append( c/(norm) )
            new_features = features._replace( counts=norm_counts )
            norm_result.append( (new_features, bed, control, shifted) )
    else:
        E.info("No normalization performed")
        norm_result = result
        
    # output sorted matrices
    if not options.sort: writeMatrix( norm_result, "unsorted" )

    for sort in options.sort: 

        if sort == "peak-height":
            norm_result.sort( key = lambda x: x[0].peak_height )
            
        elif sort == "peak-width":
            norm_result.sort( key = lambda x: x[0].peak_width )

        elif sort == "interval-width":
            norm_result.sort( key = lambda x: x[1].end - x[1].start )

        elif sort == "interval-score":
            try:
                norm_result.sort( key = lambda x: float(x[1].score) )
            except IndexError:
                E.warn("score field not present - no output" )

        writeMatrix( norm_result, sort )


def buildResults( bedfile, fg_file, control_file, counter, options ):
    '''compute densities and peakshape parameters.'''

    options.stdout.write( "\t".join( ("contig", 
                                      "start",
                                      "end",
                                      "name",
                                      "\t".join(_bam2peakshape.PeakShapeResult._fields) ) ) + "\n" )
    
    if options.window_size:
        # bins are centered at peak-center and then stretching outwards.
        bins = numpy.arange( -options.window_size + options.bin_size // 2, 
                              +options.window_size, 
                              options.bin_size )        
        
    #contigs = set(pysam_in.references)
 
    strand_specific = options.strand_specific

    result =[]
    c = E.Counter()
    c.input = 0

    for bed in Bed.iterator( IOTools.openFile( bedfile ) ):
        c.input += 1

        #if bed.contig not in contigs: 
        #    c.skipped += 1
        #    continue

        if c.input % options.report_step == 0:
            E.info( "iteration: %i" % c.input )
        
        features = counter.countInInterval( fg_file, bed.contig, bed.start, bed.end, 
                                            window_size = options.window_size,
                                            bins = bins,
                                            only_interval = options.only_interval,
                                            centring_method = options.centring_method )
     

        if control_file:
            control = counter.countAroundPos(control_file, 
                                             bed.contig,
                                             features.peak_center,
                                             bins = features.bins )

        else:
            control = None

        if options.random_shift:
            direction = numpy.random.randint( 0, 2 )
            if direction: pos = features.peak_center + 2 * bins[0]
            else: pos = features.peak_center + 2 * bins[-1]
            shifted = counter.countAroundPos(fg_file, 
                                             bed.contig,
                                             pos,
                                             bins = features.bins )
        else:
            shifted = None

        if strand_specific and bed.strand == "-":
            features._replace( hist=hist[::-1] )
            if control: control._replace( hist=hist[::-1] )
            if shifted: shift._replace( hist=hist[::-1] )

        result.append( (features, bed, control, shifted) )
        c.added += 1

    E.info( "interval processing: %s" % c )

    if c.input == 0:
        E.warn( "no data - no output" )
        E.Stop()
        return
    
    return result, bins

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    parser = buildOptionParser( argv )
    
    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    if len(args) != 2:
        raise ValueError("please specify a bam and bed file" )

    infile, bedfile = args
    control_file = None
    if options.control_file:
        E.info("using control file %s" % options.control_file)

    if options.format == "bigwig":
        fg_file = bx.bbi.bigwig_file.BigWigFile( open(infile) )
        if options.control_file:
            control_file = bx.bbi.bigwig_file.BigWigFile( open(options.control_file) )
        counter = _bam2peakshape.CounterBigwig()

    elif options.format == "bam":
        fg_file = pysam.Samfile( infile, "rb" )
        if options.control_file:
            control_file = pysam.Samfile( options.control_file, "rb" )
        counter = _bam2peakshape.CounterBam( shift = options.shift)

    result, bins = buildResults( bedfile,
                                 fg_file, 
                                 control_file, 
                                 counter,
                                 options )

    if len(result) == 0:
        E.warn( "no data - no output" )
        E.Stop()
        return

    outputResults( result, bins, options )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
add_cgat_script_template.py - add template to python scipts
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

adds a preamble to python scripts. The original scripts
are saved as ``.bak`` files.

Usage
-----

Example::

   python add_cgat_script_template.py script1.py script2.py

Type::

   python add_cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import shutil

import CGAT.Experiment as E

SCRIPT = """################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
%(filename)s - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python %(filename)s --help

Type::

   python %(filename)s --help

for command line help.

Documentation
-------------

Code
----

'''
"""

MODULE = """################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
%(filename)s - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
"""

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--template", dest="template", type="choice",
                      choices = ("script", "module"),
                      help="which template to choose [default=%default]."  )

    parser.set_defaults(
        template = "script",
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if options.template == "script":
        template = SCRIPT
    elif options.template == "module":
        template = MODULE

    for filename in args:
        E.info( "processing %s" % filename)
        lines = open(filename).readlines()
        x = 0
        while x < len(lines) and lines[x][0] in ("#", "", "\n"):
            x += 1
        l = lines[x]
        if l.startswith( "'''" ) or l.startswith('"""'): continue
        E.info( "moving %s to %s.bak" % (filename, filename) )
        shutil.move( filename, "%s.bak" % filename )
        outfile = open( filename, "w")
        outfile.write( template % locals() )
        outfile.write( "".join(lines[x:]) )
        outfile.close()

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""Sockets.py - working with sockets
====================================

This class allows you to send variable length strings
over a socket. As I am new at this, it is probably not
efficient and 100% fault tolerant.

Look at the end of the file for example usage.
"""

import sys, os, string, getopt

import socket

class SocketException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)
    
class Socket:

    mBufferSize = 4096
    mHeaderSize = 10

    def __init__( self, sock = None):

        if sock:
            self.mSocket = sock
        else:
            self.mSocket = socket.socket( socket.AF_INET, socket.SOCK_STREAM)

    def __del__(self):
        self.Close()

    def MakeServer( self, host, port ):
        hostname = socket.gethostbyname( host )
        self.mSocket.bind( (hostname, port))
        self.mSocket.listen(1)

    def MakeClient( self, host, port ):
        hostname = socket.gethostbyname( host )
        self.mSocket = socket.socket( socket.AF_INET, socket.SOCK_STREAM)
        self.mSocket.connect( (hostname, port) )

    def SendMessage( self, msg ):
        """send atribraty length message over socket."""
        
        lmsg = len(msg)
    
        m = str(lmsg) + " " * (self.mHeaderSize - len(str(lmsg)))
        
        sent = self.mSocket.send( m )
        
        if sent != self.mHeaderSize: raise SocketException("msg header could not be sent")
        
        sent = 0
        while sent < lmsg:
            r = self.mSocket.send( msg[sent:sent+self.mBufferSize] )
            if r == 0: raise SocketException("connection broken")
            sent += r
            
    def ReceiveMessage( self ):
        """receive arbitrary length message.
        """

        msg = ""
        while len(msg) < self.mHeaderSize:
            chunk = self.mSocket.recv( self.mHeaderSize - len(msg)  )
            if chunk == "": raise SocketException("connection broken")
            msg += chunk
        
        lmsg = string.atoi(msg)
        msg = ""
        
        while lmsg > 0:
            chunk = self.mSocket.recv( min(self.mBufferSize, lmsg) )
            if chunk == "": raise SocketException("connection broken")
            msg += chunk
            lmsg -= len(chunk)
            
        return msg

    def Accept( self ):
        """wait for connection to come."""
        conn, addr = self.mSocket.accept()
        return Socket( conn ), addr

    def Close( self ):
        """close socket."""
        self.mSocket.close()
        
if __name__ == "__main__":

    param_mode = None
    param_host = "kerberos.biocenter.helsinki.fi"
    param_port = 9000
    param_loglevel = 4

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      "h:",
                                      ["host="])
        
    except getopt.error, msg:
        print USAGE
        print msg
        sys.exit(2)

    for o,a in optlist:
        if o in ("-h", "--host"):
            param_host = a
    
    if len(args) != 1:
        raise "not enough arguments specified"

    param_mode = args[0]

    if param_loglevel >= 2:
        print "# host=", param_host
        print "# port=", param_port
        print "# mode=", param_mode
        
    sock = Socket()
    
    if param_mode == "server":
        
        print "gtg_align server starting"
        sock.MakeServer( param_host, param_port )

        while 1:
            
            conn, addr = sock.Accept()
            
            msg = conn.ReceiveMessage()
            print "server received", msg

            msg += "a"
            
            conn.SendMessage( msg)
            print "server sent", msg
            
        sock.Close()

    elif param_mode == "client":
        print "client starting"        
        msg = "b"

        for x in range(1, 10):

            sock.MakeClient( param_host, param_port )

            sock.SendMessage( msg)
            print "client sent", msg

            msg = sock.ReceiveMessage()
            print "client received", msg
            
            msg += "b"
            
            sock.Close()

            



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2table.py - compute features for intersection of two gff files
=================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

collect intervals from two gff files and compute features based 
on their intersection. The script is intended to compute properties
for a set of non-overlapping windows.

Transforms:
   * none:        no transform
   * overlap:     overlap between set1 and set2
   * complement:  part of set1 that is not covered by set2
   * third_codon: only takes every third position. Needs frame information
                  in the gff file.

Decorators:
   * GC:            G+C content of intervals
   * count:         number of windows
   * mean-length:   mean length of intervals overlapping with window

Usage
-----

Example::

   python gff2table.py --help

Type::

   python gff2table.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import time
import os
import shutil
import tempfile
import math

import CGAT.Experiment as E
import CGAT.Genomics as Genomics
import CGAT.IndexedFasta as IndexedFasta
import CGAT.IndexedGenome as IndexedGenome

import CGAT.Intervals as Intervals
import CGAT.Stats as Stats
import CGAT.GTF as GTF

#########################################################
#########################################################
#########################################################
## Decorators: decorate a window through intervals
#########################################################
def decorator_counts( intervals, start, end, contig, fasta ):
    """compute length distribution."""
    d = Stats.DistributionalParameters( map( lambda x: x[1] - x[0], intervals ))
    return d['nval'], str(d)

def decorator_percent_coverage( intervals, start, end, contig, fasta ):
    """compute length of intervals."""
    d = Stats.DistributionalParameters( map( lambda x: x[1] - x[0], intervals ))
    return 100.0 * float(d['sum']) / (end - start), str(d)

def decorator_mean_length( intervals, start, end, contig, fasta ):
    """compute length distribution."""
    d = Stats.DistributionalParameters( map( lambda x: x[1] - x[0], intervals ))
    return d['mean'], str(d)

def decorator_median_length( intervals, start, end, contig, fasta ):
    """compute length distribution."""
    d = Stats.DistributionalParameters( map( lambda x: x[1] - x[0], intervals ))
    return d['median'], str(d)

def decorator_percent_gc( intervals, start, end, contig, fasta ):
    """compute G+C content in intervals.
    """
    l, ngc = 0, 0

    # fetch sequence of the complete window first
    sequence = fasta.getSequence( contig, "+", start, end )
    
    for istart, iend in intervals:
        ngc += len( filter( lambda x: x in "GCgc", sequence[istart-start:iend-start]) )
        l += iend - istart
    
    return 100.0 * ngc / l, None

def decorator_median_score( values, start, end, contig ):
    """compute median of values."""
    d = Stats.DistributionalParameters( values )
    return d['median'], str(d)

def decorator_mean_score( values, start, end, contig ):
    """compute mean of values."""
    d = Stats.DistributionalParameters( values )
    return d['mean'], str(d)

def decorator_stddev_score( values, start, end, contig ):
    """compute stddev of values."""
    d = Stats.DistributionalParameters( values )
    return d['stddev'], str(d)

def decorator_min_score( values, start, end, contig ):
    """compute minumum of values."""
    d = Stats.DistributionalParameters( values )
    return d['min'], str(d)

def decorator_max_score( values, start, end, contig ):
    """compute minumum of values."""
    d = Stats.DistributionalParameters( values )
    return d['max'], str(d)

#########################################################
#########################################################
#########################################################
## Transformers: manipulate a set of intervals
#########################################################

def transform_overlap( start, end, intervals_with_gff ):
    """transform: overlap of intervals in x with y."""
    y = Intervals.combineIntervals( map( lambda x: (x[0], x[1]), intervals_with_gff ) )
    return Intervals.pruneIntervals( y, start, end )

def transform_complement( start, end, intervals_with_gff ):
    y = Intervals.combineIntervals( map( lambda x: (x[0], x[1]), intervals_with_gff ) )
    return Intervals.complementIntervals( y, start, end )

def transform_third_codon( start, end, intervals_with_gff):
    """transform: only return nucleotide positions in window (start, end) 
    that are in third codon position.
    """
    intervals = []
    for istart, iend, gff in intervals_with_gff: 

        if gff.frame == ".":
            raise ValueError("need a frame for third codon positions.")
        
        # frame = nucleotides from start to next codon
        frame = int(gff.frame)

        # to make life easier, convert to 0-based coordinates, 
        # with zero starting at first position in window
        # re-arrange positions on negative strand
        if Genomics.IsNegativeStrand( gff.strand ):
            # convert to negative strand coordinates counting from 0
            coordinate_offset = end
            reverse = True
            istart, iend = end - iend, end - istart
        else: 
            istart, iend = istart - start, iend - start
            reverse = False
            coordinate_offset = start
            
        # make sure that you start on a second codon position and within window
        if istart < 0:
            frame = (frame + istart) % 3
            istart = 0
        if frame != 0: istart -= (3-frame)
        istart += 2

        iend = min( iend, end - start)

        for x in range( istart, iend, 3 ):

            if reverse:
                c = coordinate_offset - x - 1
            else:
                c = coordinate_offset + x
            intervals.append( (c, c+1) )

    return Intervals.combineIntervals( intervals )

def test_transform_third_codon():

    def test_entry( frame, strand, xfrom, xto, start, end, ref ):

        entry = GTF.Entry()
        entry.frame = frame
        entry.strand = strand
        entry.start = xfrom
        entry.end = xto
        
        intervals = transform_third_codon( start, end, [ (xfrom, xto, entry) ] )
        if ref != intervals:
            print "failed:", ref != intervals
        
    test_entry( 0, "+", 1, 7, 0, 6, [(3,4)] )
    test_entry( 0, "-", 1, 7, 0, 6, [(1,2), (4,5)])
    test_entry( 1, "+", 1, 7, 0, 6, [(1,2), (4,5)])
    test_entry( 2, "+", 1, 7, 0, 6, [(2,3), (5,6)])
    test_entry( 1, "-", 1, 7, 0, 6, [(3,4)] )
    test_entry( 2, "-", 1, 7, 0, 6, [(2,3), (5,6)] )

    sys.exit(0)

def annotateWindows( contig, windows, gff_data, fasta, options ):
    """annotate windows."""

    index = IndexedGenome.IndexedGenome()
    for g in gff_data:
        index.add( g.contig, g.start, g.end, g )
    
    is_gtf = options.is_gtf

    if options.transform == "none":
        transform = lambda x,y,z: map( lambda x: (x[0], x[1]), z)
    elif options.transform == "overlap":
        transform = transform_overlap
    elif options.transform == "complement":
        transform = transform_complement
    elif options.transform == "third_codon":
        transform = transform_third_codon
    else:
        raise ValueError("unknown transform %s" % options.transform)

    work_on_intervals = True
    if options.decorator == "counts":
        decorator = decorator_counts
    elif options.decorator == "mean-length":
        decorator = decorator_mean_length
    elif options.decorator == "median-length":
        decorator = decorator_median_length
    elif options.decorator == "percent-coverage":
        decorator = decorator_percent_coverage
    elif options.decorator == "gc":
        decorator = decorator_percent_gc
    elif options.decorator == "median-score":
        decorator = decorator_median_score
        work_on_intervals = False
    elif options.decorator == "mean-score":
        decorator = decorator_mean_score
        work_on_intervals = False
    elif options.decorator == "stddev-score":
        decorator = decorator_stddev_score
        work_on_intervals = False
    elif options.decorator == "min-score":
        decorator = decorator_min_score
        work_on_intervals = False
    elif options.decorator == "max-score":
        decorator = decorator_max_score
        work_on_intervals = False
    else:
        raise ValueError("unknown decorator %s" % options.decorator)

    for start,end in windows:
        
        # counts/length before/after transformation
        n1, l1, n2, l2 = 0, 0, 0, 0

        values, intervals_with_gff, genes, transcripts = [], [], set(), set()
        
        try:
            for istart, iend, value in index.get( contig, start, end ):
                n1 += 1
                l1 += iend - istart
                intervals_with_gff.append( (istart, iend, value) )
                values.append( value.score )
                if is_gtf:
                    genes.add( value.gene_id )
                    transcripts.add( value.transcript_id )
        except KeyError:
            pass
        
        if n1 == 0 and options.skip_empty: continue

        if work_on_intervals:

            if options.loglevel >= 3:
                options.stdlog.write("# intervals in window %i:%i before transformation: %s\n" % (start, end, str(intervals)))

            intervals = transform( start, end, intervals_with_gff )

            for xstart, xend in intervals:
                n2 += 1
                l2 += xend - xstart

            if options.loglevel >= 3:
                options.stdlog.write("# intervals in window %i:%i after transformation: %s\n" % (start,end, str(intervals)))

            score, extra_info = decorator( intervals, start, end, contig, fasta )
            
        else:
            if len(values) > 0:
                values = map(float, values)
                score, extra_info = decorator( values, start, end, contig )
            else:
                score, extra_info = 0, None
                
            l2 = 0
            n2 = 0

        if is_gtf:
            ngenes, ntranscripts = len(genes), len(transcripts)
        else:
            ngenes, ntranscripts = 0, 0

        if extra_info:
            extra_info = re.sub("\t", ";", extra_info )
        options.stdout.write( "\t".join( \
            map(str, (contig, start, end,
                      ngenes, ntranscripts,
                      n1, l1,
                      n2, l2,
                      score,
                      extra_info) ) ) + "\n" )
                      
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2table.py 2861 2010-02-23 17:36:32Z andreas $", 
                                    usage = globals()["__doc__"])

    parser.add_option( "-g", "--genome-file", dest="genome_file", type="string",
                       help="filename with genome (indexed)."  )

    parser.add_option( "-w", "--filename-windows", dest="filename_windows", type="string",
                       help="gff file with windows to use."  )

    parser.add_option( "-d", "--filename-data=", dest="filename_data", type="string",
                       help="gff file with data to use." )

    parser.add_option( "--is-gtf", dest="is_gtf", action="store_true",
                       help="filename-data is gtf file [default=%default." )

    parser.add_option( "-f", "--features=", dest="features", type="choice", action="append",
                       choices=("GC", ),
                       help="features to compute." )

    parser.add_option( "-c", "--decorator=", dest="decorator", type="choice", 
                       choices=( "counts", "gc", "gc3", "mean-length", "median-length", "percent-coverage",
                                 "median-score", "mean-score", "stddev-score", "min-score", "max-score" ),
                       help="decorators to use." )

    parser.add_option( "-e", "--skip-empty", dest="skip_empty", action="store_true",
                       help="skip empty windows." )

    parser.add_option( "-t", "--transform=", dest="transform", type="choice", 
                       choices=("none", "overlap", "complement", "third_codon"),
                       help="transform to use when mapping overlapping regions onto window." )


    parser.set_defaults(
        genome_file = None,
        filename_windows = None,
        filename_data = None,
        features = [],
        skip_empty = False,
        decorator = "counts",
        transform = "none",
        is_gtf = False,
        )

    (options, args) = E.Start( parser )

    #    test_transform_third_codon()
    
    if not options.filename_windows:
        raise ValueError("please supply a gff file with window information." )
    
    if options.loglevel >= 1:
        options.stdlog.write("# reading windows..." )
        options.stdlog.flush()
        
    windows = GTF.readAsIntervals( GFF.iterator( IOTools.openFile(options.filename_windows, "r" ) ) )

    if options.loglevel >= 1:
        options.stdlog.write("done\n" )
        options.stdlog.flush()

    if options.filename_data:
        if options.loglevel >= 1:
            options.stdlog.write("# reading data..." )
            options.stdlog.flush()

        if options.is_gtf:
            gff_data = GTF.readFromFile( IOTools.openFile( options.filename_data, "r" ) )
        else: 
            gff_data = GTF.readFromFile( IOTOols.openFile( options.filename_data, "r" ) )

        if options.loglevel >= 1:
            options.stdlog.write("done\n" )
            options.stdlog.flush()
        
        data_ranges = GTF.SortPerContig( gff_data )
    else:
        ## use windows to compute properties
        ## by supplying no data and asking for the complement = original window
        gff_data = None
        data_ranges = None
        options.transform = "complement"

    map_contig2size = {}

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
        map_contig2size = fasta.getContigSizes()
    else:
        for contig, values in windows.items():
            map_contig2size[contig] = max( lambda x:x[1], values )
        fasta = None

    contigs = map_contig2size.keys()
    contigs.sort()

    ## proceed contig wise
    noutput_contigs, ncontigs_skipped_windows, ncontigs_skipped_data = 0, 0, 0

    options.stdout.write( "\t".join( \
        map(str, ("contig", "start", "end",
                  "ngenes", "ntranscripts",
                  "n1", "l1",
                  "n2", "l2",
                  "score",
                  "extra_info") ) ) + "\n" )
    
    for contig in contigs:
        
        skip = False
        if contig not in windows: 
            ncontigs_skipped_windows += 1
            skip = True

        if data_ranges and contig not in data_ranges: 
            ncontigs_skipped_data += 1
            skip = True
            
        if skip: continue
        
        noutput_contigs += 1 
        if data_ranges:
            annotateWindows( contig,
                             windows[contig],
                             gff_data[data_ranges[contig][0]:data_ranges[contig][1]],
                             fasta,
                             options )
        else:
            annotateWindows( contig,
                             windows[contig],
                             [],
                             fasta,
                             options )
            

    if options.loglevel >= 1:
        options.stdout.write( "# ninput_windows=%i, noutput_contigs=%i, ninput_contigs=%i, nskipped_windows=%i, nskipped_data=%i\n" %\
                                  ( len(windows), noutput_contigs, len(contigs), ncontigs_skipped_windows, ncontigs_skipped_data ) )

    E.Stop()





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
diff_fasta.py - compare contents of two fasta files
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes two sets of fasta sequences and matches the
identifiers. It then comparse the sequences associated with the
identifiers and outputs

   * which sequences are missing
   * which sequences are identical
   * which sequences are prefixes/suffixes of each other

Depending on the option ``--output`` the following are output:

diff
   identifiers of sequences that are different

seqdiff
   sequences of sequences that are different

missed
   seqences that are missing from set or the other

Usage
-----

Example::

   python diff_fasta.py a.fasta b.fasta

Type::

   python diff_fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import optparse

import CGAT.Genomics as Genomics
import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def MapIdentifiers( seqs, pattern):

    rx = re.compile(pattern)
    
    for k,s in seqs.items():
        try:
            nk = rx.search(k).groups()[0]
        except AttributeError:
            raise ValueError( "identifier can not be parsed from '%s' pattern='%s'" % (k, pattern) )
            
        del seqs[k]
        seqs[nk] = s
        
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: diff_fasta.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-s", "--correct-gap-shift", dest="correct_shift", action="store_true",
                      help="correct gap length shifts in alignments. Requires alignlib. "
                      "[%default]")
    parser.add_option("-1", "--pattern1", dest="pattern1", type="string",
                      help="pattern to extract identifier from in identifiers1. "
                      "[%default]")
    parser.add_option("-2", "--pattern2", dest="pattern2", type="string",
                      help="pattern to extract identifier from in identifiers2. "
                      "[%default]")

    parser.add_option("-o", "--output", dest="output", type="choice", action="append",
                      choices=("diff", "missed", "seqdiff"),
                      help="what to output [%default]")
                      
    parser.set_defaults( correct_shift = False,
                         pattern1 = "(\S+)",
                         pattern2 = "(\S+)",
                         output = [] ) 
                         

    (options, args) = E.Start( parser )

    if len(args) != 2:
        raise ValueError( "two files needed to compare." )

    if options.correct_shift:
        try:
            import alignlib
        except ImportError:
            raise ImportError("option --correct-shift requires alignlib, but alignlib not found" )

    seqs1 = Genomics.ReadPeptideSequences( IOTools.openFile(args[0], "r") )
    seqs2 = Genomics.ReadPeptideSequences( IOTools.openFile(args[1], "r") )    

    if not seqs1: raise ValueError( "first file %s is empty." % (args[0] ) )
    if not seqs2: raise ValueError( "second file %s is empty." % (args[1] ) )
    
    MapIdentifiers( seqs1, options.pattern1 )
    MapIdentifiers( seqs2, options.pattern2 )    
    
    nsame = 0
    nmissed1 = 0
    nmissed2 = 0
    ndiff = 0
    ndiff_first = 0
    ndiff_last = 0
    ndiff_prefix = 0
    ndiff_selenocysteine = 0
    ndiff_masked = 0
    nfixed = 0
    found2 = {}

    write_missed1 = "missed" in options.output
    write_missed2 = "missed" in options.output
    write_seqdiff = "seqdiff" in options.output
    write_diff = "diff" in options.output or write_seqdiff
    
    for k in seqs1:
        if k not in seqs2:
            nmissed1+=1
            if write_missed1:
                options.stdout.write("---- %s ---- %s\n" % (k, "missed1") )
            continue
        
        found2[k] = 1
        
        s1 = seqs1[k].upper()
        s2 = seqs2[k].upper()
        m = min(len(s1), len(s2))
        
        if s1 == s2:
            nsame += 1
        else:
            status = "other"
            
            ndiff += 1
            
            if s1[1:] == s2[1:]:
                ndiff_first += 1
                status = "first"
            elif s1[:m] == s2[:m]:
                ndiff_prefix += 1
                status = "prefix"
            elif s1[:-1] == s2[:-1]:
                ndiff_last += 1
                status = "last"
            else:
                if len(s1) == len(s2):
                    # get all differences:
                    # the first and last residues can be different for peptide sequences when comparing
                    # my translations with ensembl peptides.
                    differences = []
                    for x in range(1,len(s1)-1):
                        if s1[x] != s2[x]:
                            differences.append( (s1[x], s2[x]) )

                    l = len(differences)
                    # check for Selenocysteins
                    if len( filter( lambda x: x[0] == "U" or x[1] == "U", differences)) == l:
                        ndiff_selenocysteine += 1
                        status = "selenocysteine"
                        
                    # check for masked residues
                    elif len( filter( lambda x: x[0] in "NX" or x[1] in "NX", differences)) == l:
                        ndiff_masked += 1
                        status = "masked"
                        
            ## correct for different gap lengths
            if options.correct_shift:
                        
                map_a2b = alignlib.makeAlignmentVector()

                a, b = 0, 0
                keep = False
                
                x = 0
                while x < m and not (a == len(s1) and b == len(s2)):
                    try:
                        if s1[a] != s2[b]:
                            while s1[a] == "N" and s2[b] != "N":
                                a += 1
                            while s1[a] != "N" and s2[b] == "N":
                                b += 1
                            
                            if s1[a] != s2[b]:
                                break
                    except IndexError:
                        print "# index error for %s: x=%i, a=%i, b=%i, l1=%i, l2=%i" % (k, x, a, b, len(s1), len(s2))
                        break

                    a += 1
                    b += 1
                    map_a2b.addPairExplicit( a, b, 0.0 )
                    ## check if we have reached the end:
                else:
                    keep = True
                    nfixed += 1
                    f = alignlib.AlignmentFormatEmissions( map_a2b)
                    print "fix\t%s\t%s" % (k, str(f))

                if not keep:
                    print "# warning: not fixable: %s" % k
                    
            if write_diff:
                options.stdout.write("---- %s ---- %s\n" % (k, status) )

            if write_seqdiff:
                options.stdout.write( "< %s\n> %s\n" % (seqs1[k], seqs2[k]) )

    for k in seqs2.keys():
        if k not in found2:
            nmissed2 += 1
            if write_missed2:
                options.stdout.write("---- %s ---- %s\n" % (k, "missed2") )

    options.stdlog.write( """# Legend:
# seqs1:          number of sequences in set 1
# seqs2:          number of sequences in set 2
# same:           number of identical sequences
# diff:           number of sequences with differences
# nmissed1:       sequences in set 1 that are not found in set 2
# nmissed2:       sequences in set 2 that are not found in set 1
# Type of sequence differences
# first:          only the first residue is different
# last:           only the last residue is different
# prefix:         one sequence is prefix of the other
# selenocysteine: difference due to selenocysteines
# masked:         difference due to masked residues
# fixed:          fixed differences
# other:          other differences
""")

    E.info( "seqs1=%i, seqs2=%i, same=%i, ndiff=%i, nmissed1=%i, nmissed2=%i" %\
                (len(seqs1), len(seqs2), nsame, ndiff, nmissed1, nmissed2) )
    E.info( "ndiff=%i: first=%i, last=%i, prefix=%i, selenocysteine=%i, masked=%i, fixed=%i, other=%i" %\
                (ndiff, ndiff_first, ndiff_last, ndiff_prefix, ndiff_selenocysteine, ndiff_masked, nfixed,
                 ndiff - ndiff_first - ndiff_last - ndiff_prefix - ndiff_selenocysteine - ndiff_masked - nfixed) )
    
    E.Stop()
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
csv_cut.py - select columns from a table
========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

extract named columns from a csv formatted table


.. todo::
   
   describe purpose of the script.

Usage
-----

Extract the two columns gene and length from a table in standard input::

   python csv_cut.py gene length < stdin

The script permits the use of patterns. For example, the command 
will select the column gene and all columns that contain the part 'len'::

   python csv_cut.py gene %len% < stdin

Type::

   python csv_cut.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

import CGAT.Experiment as E
import csv
import _csv
import hashlib
import CGAT.CSV as CSV

USAGE="""csv_cut.py [OPTIONS] col1 [col2 [...]] < stdin

"""

class UniqueBuffer:
    mKeys = {}
    def __init__(self, outfile):
        self.mOutfile = outfile
    def write( self, out ):
        key = hashlib.md5(out).digest()
        if key not in self.mKeys:
            self.mKeys[key] = True
            self.mOutfile.write(out)

class CommentStripper:
    """iterator class for stripping comments from file.
    """
    
    def __init__(self, file ):
        self.mFile = file

    def __iter__(self):
        return self
    
    def next(self):
        while 1:
            line = self.mFile.readline()
            if not line:
                raise StopIteration
            if line[0] != "#":
                return line
        
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: csv_cut.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-r", "--remove", dest="remove", action="store_true",
                       help="remove specified columns, keep all others." )

    parser.add_option( "-u", "--unique", dest="unique", action="store_true",
                       help="output rows are uniq." )

    parser.add_option( "-l", "--large", dest="large", action="store_true",
                       help="large columns. Do not use native python CSV module [default=%default]." )

    parser.add_option( "-f", "--filename-fields", dest="filename_fields", type="string",
                       help="filename with field information.")

    parser.set_defaults(
        remove = False,
        unique = False,
        filename_fields = None,
        )

    (options, args) = E.Start( parser,
                               add_csv_options  = True,
                               quiet = True )

    input_fields = args

    if options.filename_fields:
        input_fields = map( lambda x: x[:-1].split("\t")[0],
                            filter( lambda x: x[0] != "#", open(options.filename_fields, "r").readlines()))

                            
    if options.unique:
        outfile = UniqueBuffer(sys.stdout)
    else:
        outfile = options.stdout

    while 1:
        line = sys.stdin.readline()
        
        if not line:
            E.Stop()
            sys.exit(0)
        
        if line[0] == "#": continue
        
        first_line = line
        break
            
    old_fields = first_line[:-1].split("\t")

    fields = []
    for f in input_fields:
        ## do pattern search
        if f[0] == "%" and f[-1] == "%":
            pattern = re.compile( f[1:-1] )
            for o in old_fields:
                if pattern.search( o ) and o not in fields:
                    fields.append( o )
        else:
            if f in old_fields:
                fields.append( f )
    
    if options.remove:
        fields = set(fields)
        fields = [ x for x in old_fields if x not in fields ]
    
    if options.large:
        reader = CSV.DictReaderLarge( CommentStripper(sys.stdin),
                                      fieldnames = old_fields,
                                      dialect=options.csv_dialect )
    else:
        reader = csv.DictReader( CommentStripper(sys.stdin),
                                 fieldnames = old_fields,
                                 dialect=options.csv_dialect )

    writer = csv.DictWriter( outfile,
                             fields,
                             dialect=options.csv_dialect,
                             lineterminator = options.csv_lineterminator,
                             extrasaction = 'ignore' )

    print "\t".join(fields)        

    first_row = True
    ninput, noutput, nerrors = 0, 0, 0

    while 1:
        ninput += 1
        try:
            row = reader.next()
        except _csv.Error, msg:
            options.stderr.write( "# error while parsing: %s\n" % (msg ) )
            nerrors += 1
            continue
        except StopIteration:
            break
        if not row: break
        writer.writerow(row)
        noutput += 1

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nerrors=%i\n" % (ninput, noutput, nerrors) )

    E.Stop()
